@@ -72,20 +72,6 @@ if mxdatetime_available:
 
 creationOrder = count()
 
-class SQLValidator(compound.All):
-    def attemptConvert(self, value, state, validate):
-        if validate is validators.to_python:
-            vlist = list(self.validators[:])
-            vlist.reverse()
-        elif validate is validators.from_python:
-            vlist = self.validators
-        else:
-            raise RuntimeError
-        for validator in vlist:
-            value = validate(validator, value, state)
-        return value
-
-
 ########################################
 ## Columns
 ########################################
@@ -209,7 +195,7 @@ class SOCol(object):
         _validators = self.createValidators()
         if _validators:
             if validator: _validators.append(validator)
-            self.validator = SQLValidator.join(_validators[0], *_validators[1:])
+            self.validator = compound.All.join(_validators[0], *_validators[1:])
         else:
             self.validator = validator
         self.noCache = noCache
@@ -512,27 +498,31 @@ class StringValidator(validators.Validator):
     def to_python(self, value, state):
         if value is None:
             return None
-        if isinstance(value, unicode):
-            connection = state.soObject._connection
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            dbEncoding = "ascii"
+            binaryType = type(None) # Just a simple workaround
+        else:
             dbEncoding = getattr(connection, "dbEncoding", None) or "ascii"
+            binaryType = connection._binaryType
+        if isinstance(value, unicode):
             return value.encode(dbEncoding)
-        return value
-
-    def from_python(self, value, state):
-        if value is None:
-            return None
-        if isinstance(value, str):
+        if self.dataType and isinstance(value, self.dataType):
             return value
-        if isinstance(value, unicode):
-            connection = state.soObject._connection
-            dbEncoding = getattr(connection, "dbEncoding", None) or "ascii"
-            return value.encode(dbEncoding)
-        return value
+        if isinstance(value, (str, buffer, binaryType, sqlbuilder.SQLExpression)):
+            return value
+        if hasattr(value, '__unicode__'):
+            return unicode(value).encode(dbEncoding)
+        raise validators.Invalid("expected a str in the StringCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
+
+    from_python = to_python
 
 class SOStringCol(SOStringLikeCol):
 
-    def createValidators(self):
-        return [StringValidator()] + \
+    def createValidators(self, dataType=None):
+        return [StringValidator(name=self.name, dataType=dataType)] + \
             super(SOStringCol, self).createValidators()
 
 class StringCol(Col):
@@ -543,18 +533,28 @@ class UnicodeStringValidator(validators.Validator):
     def to_python(self, value, state):
         if value is None:
             return None
-        if isinstance(value, unicode):
+        if isinstance(value, (unicode, sqlbuilder.SQLExpression)):
             return value
+        if isinstance(value, str):
+            return unicode(value, self.dbEncoding)
         if isinstance(value, array): # MySQL
-            return unicode(value.tostring(), self.db_encoding)
-        return unicode(value, self.db_encoding)
+            return unicode(value.tostring(), self.dbEncoding)
+        if hasattr(value, '__unicode__'):
+            return unicode(value)
+        raise validators.Invalid("expected a str or a unicode in the UnicodeCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
     def from_python(self, value, state):
         if value is None:
             return None
-        if isinstance(value, str):
+        if isinstance(value, (str, sqlbuilder.SQLExpression)):
             return value
-        return value.encode(self.db_encoding)
+        if isinstance(value, unicode):
+            return value.encode(self.dbEncoding)
+        if hasattr(value, '__unicode__'):
+            return unicode(value).encode(self.dbEncoding)
+        raise validators.Invalid("expected a str or a unicode in the UnicodeCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
 class SOUnicodeCol(SOStringLikeCol):
     def __init__(self, **kw):
@@ -562,7 +562,8 @@ class SOUnicodeCol(SOStringLikeCol):
         super(SOUnicodeCol, self).__init__(**kw)
 
     def createValidators(self):
-        return [UnicodeStringValidator(db_encoding=self.dbEncoding)] + \
+        return [UnicodeStringValidator(name=self.name,
+                dbEncoding=self.dbEncoding)] + \
             super(SOUnicodeCol, self).createValidators()
 
 class UnicodeCol(Col):
@@ -576,19 +577,16 @@ class IntValidator(validators.Validator):
             return None
         if isinstance(value, (int, long, sqlbuilder.SQLExpression)):
             return value
-        try:
-            return int(value)
-        except:
-            raise validators.Invalid("expected an int in the IntCol '%s', got %s %r instead" % \
+        for converter, attr_name in (int, '__int__'), (long, '__long__'):
+            if hasattr(value, attr_name):
+                try:
+                    return converter(value)
+                except:
+                    break
+        raise validators.Invalid("expected an int in the IntCol '%s', got %s %r instead" % \
                 (self.name, type(value), value), value, state)
 
-    def from_python(self, value, state):
-        if value is None:
-            return None
-        if not isinstance(value, (int, long, sqlbuilder.SQLExpression)):
-            raise validators.Invalid("expected an int in the IntCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
-        return value
+    from_python = to_python
 
 class SOIntCol(SOCol):
     # 3-03 @@: support precision, maybe max and min directly
@@ -658,25 +656,21 @@ class BoolValidator(validators.Validator):
     def to_python(self, value, state):
         if value is None:
             return None
-        elif not value:
-            return False
-        else:
-            return True
+        if isinstance(value, (bool, sqlbuilder.SQLExpression)):
+            return value
+        if isinstance(value, (int, long)) or hasattr(value, '__nonzero__'):
+            return bool(value)
+        raise validators.Invalid("expected a bool or an int in the BoolCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
-    def from_python(self, value, state):
-        if value is None:
-            return None
-        elif value:
-            return True
-        else:
-            return False
+    from_python = to_python
 
 class SOBoolCol(SOCol):
     def autoConstraints(self):
         return [consts.isBool]
 
     def createValidators(self):
-        return [BoolValidator()] + \
+        return [BoolValidator(name=self.name)] + \
             super(SOBoolCol, self).createValidators()
 
     def _postgresType(self):
@@ -709,21 +703,18 @@ class FloatValidator(validators.Validator):
     def to_python(self, value, state):
         if value is None:
             return None
-        if isinstance(value, (int, long, float, sqlbuilder.SQLExpression)):
+        if isinstance(value, (float, int, long, sqlbuilder.SQLExpression)):
             return value
-        try:
-            return float(value)
-        except:
-            raise validators.Invalid("expected a float in the FloatCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
+        for converter, attr_name in  (float, '__float__'), (int, '__int__'), (long, '__long__'):
+            if hasattr(value, attr_name):
+                try:
+                    return converter(value)
+                except:
+                    break
+        raise validators.Invalid("expected a float in the FloatCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
-    def from_python(self, value, state):
-        if value is None:
-            return None
-        if not isinstance(value, (int, long, float, sqlbuilder.SQLExpression)):
-            raise validators.Invalid("expected a float in the FloatCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
-        return value
+    from_python = to_python
 
 class SOFloatCol(SOCol):
     # 3-03 @@: support precision (e.g., DECIMAL)
@@ -922,12 +913,10 @@ class EnumValidator(validators.Validator):
             return value
         elif not self.notNone and value is None:
             return None
-        else:
-            raise validators.Invalid("expected a member of %r in the EnumCol '%s', got %r instead" % \
-                                     (self.enumValues, self.name, value), value, state)
+        raise validators.Invalid("expected a member of %r in the EnumCol '%s', got %r instead" % \
+            (self.enumValues, self.name, value), value, state)
 
-    def from_python(self, value, state):
-        return self.to_python(value, state)
+    from_python = to_python
 
 class SOEnumCol(SOCol):
 
@@ -997,15 +986,19 @@ class SetValidator(validators.Validator):
     """
 
     def to_python(self, value, state):
-        if not isinstance(value, str):
-            raise validators.Invalid("expected a value string, got % instead" % \
-                                      (value), value, state)
-        return tuple(value.split(","))
+        if isinstance(value, str):
+            return tuple(value.split(","))
+        raise validators.Invalid("expected a string in the SetCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
     def from_python(self, value, state):
-        if not isinstance(value, tuple):
+        if isinstance(value, basestring):
             value = (value,)
-        return ",".join([v for v in value])
+        try:
+            return ",".join(value)
+        except:
+            raise validators.Invalid("expected a string or a sequence of stringsin the SetCol '%s', got %s %r instead" % \
+                (self.name, type(value), value), value, state)
 
 class SOSetCol(SOCol):
     def __init__(self, **kw):
@@ -1154,7 +1147,7 @@ class DateValidator(DateTimeValidator):
     def to_python(self, value, state):
         if isinstance(value, datetime.datetime):
             value = value.date()
-        if isinstance(value, datetime.date):
+        if isinstance(value, (datetime.date, sqlbuilder.SQLExpression)):
             return value
         value = super(DateValidator, self).to_python(value, state)
         if isinstance(value, datetime.datetime):
@@ -1212,7 +1205,7 @@ class DateCol(Col):
 
 class TimeValidator(DateTimeValidator):
     def to_python(self, value, state):
-        if isinstance(value, datetime.time):
+        if isinstance(value, (datetime.time, sqlbuilder.SQLExpression)):
             return value
         if isinstance(value, datetime.timedelta):
             if value.days:
@@ -1225,6 +1218,8 @@ class TimeValidator(DateTimeValidator):
             value = value.time()
         return value
 
+    from_python = to_python
+
 class SOTimeCol(SOCol):
     timeFormat = '%H:%M:%S'
 
@@ -1283,6 +1278,24 @@ class TimestampCol(Col):
     baseClass = SOTimestampCol
 
 
+class TimedeltaValidator(validators.Validator):
+    def to_python(self, value, state):
+        return value
+
+    from_python = to_python
+
+class SOTimedeltaCol(SOCol):
+    def _postgresType(self):
+        return 'INTERVAL'
+
+    def createValidators(self):
+        return [TimedeltaValidator(name=self.name)] + \
+            super(SOTimedeltaCol, self).createValidators()
+
+class TimedeltaCol(Col):
+    baseClass = SOTimedeltaCol
+
+
 from decimal import Decimal
 
 class DecimalValidator(validators.Validator):
@@ -1293,9 +1306,13 @@ class DecimalValidator(validators.Validator):
             return value
         if isinstance(value, float):
             value = str(value)
-        connection = state.soObject._connection
-        if hasattr(connection, "decimalSeparator"):
-            value = value.replace(connection.decimalSeparator, ".")
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            pass
+        else:
+            if hasattr(connection, "decimalSeparator"):
+                value = value.replace(connection.decimalSeparator, ".")
         try:
             return Decimal(value)
         except:
@@ -1308,18 +1325,22 @@ class DecimalValidator(validators.Validator):
         if isinstance(value, float):
             value = str(value)
         if isinstance(value, basestring):
-            connection = state.soObject._connection
-            if hasattr(connection, "decimalSeparator"):
-                value = value.replace(connection.decimalSeparator, ".")
+            try:
+                connection = state.connection or state.soObject._connection
+            except AttributeError:
+                pass
+            else:
+                if hasattr(connection, "decimalSeparator"):
+                    value = value.replace(connection.decimalSeparator, ".")
             try:
                 return Decimal(value)
             except:
                 raise validators.Invalid("can not parse Decimal value '%s' in the DecimalCol from '%s'" %
                     (value, getattr(state, 'soObject', '(unknown)')), value, state)
-        if not isinstance(value, (int, long, Decimal, sqlbuilder.SQLExpression)):
-            raise validators.Invalid("expected a decimal in the DecimalCol '%s', got %s %r instead" % \
-                (self.name, type(value), value), value, state)
-        return value
+        if isinstance(value, (int, long, Decimal, sqlbuilder.SQLExpression)):
+            return value
+        raise validators.Invalid("expected a Decimal in the DecimalCol '%s', got %s %r instead" % \
+            (self.name, type(value), value), value, state)
 
 class SODecimalCol(SOCol):
 
@@ -1390,12 +1411,13 @@ class SODecimalStringCol(SOStringCol):
 
     def createValidators(self):
         if self.quantize:
-            v = DecimalStringValidator(
+            v = DecimalStringValidator(name=self.name,
                 precision=Decimal(10) ** (-1 * int(self.precision)),
                 max=Decimal(10) ** (int(self.size) - int(self.precision)))
         else:
-            v = DecimalStringValidator(precision=0)
-        return [v] + super(SODecimalStringCol, self).createValidators()
+            v = DecimalStringValidator(name=self.name, precision=0)
+        return [v] + \
+            super(SODecimalStringCol, self).createValidators(dataType=Decimal)
 
 class DecimalStringCol(StringCol):
     baseClass = SODecimalStringCol
@@ -1414,12 +1436,19 @@ class BinaryValidator(validators.Validator):
     def to_python(self, value, state):
         if value is None:
             return None
+        try:
+            connection = state.connection or state.soObject._connection
+        except AttributeError:
+            dbName = None
+            binaryType = type(None) # Just a simple workaround
+        else:
+            dbName = connection.dbName
+            binaryType = connection._binaryType
         if isinstance(value, str):
-            connection = state.soObject._connection
-            if connection.dbName == "sqlite":
+            if dbName == "sqlite":
                 value = connection.module.decode(value)
             return value
-        if isinstance(value, (buffer, state.soObject._connection._binaryType)):
+        if isinstance(value, (buffer, binaryType)):
             cachedValue = self._cachedValue
             if cachedValue and cachedValue[1] == value:
                 return cachedValue[0]
@@ -1432,7 +1461,8 @@ class BinaryValidator(validators.Validator):
     def from_python(self, value, state):
         if value is None:
             return None
-        binary = state.soObject._connection.createBinary(value)
+        connection = state.connection or state.soObject._connection
+        binary = connection.createBinary(value)
         self._cachedValue = (value, binary)
         return binary
 
@@ -1484,8 +1514,12 @@ class PickleValidator(BinaryValidator):
         if value is None:
             return None
         if isinstance(value, unicode):
-            connection = state.soObject._connection
-            dbEncoding = getattr(connection, "dbEncoding", None) or "ascii"
+            try:
+                connection = state.connection or state.soObject._connection
+            except AttributeError:
+                dbEncoding = "ascii"
+            else:
+                dbEncoding = getattr(connection, "dbEncoding", None) or "ascii"
             value = value.encode(dbEncoding)
         if isinstance(value, str):
             return pickle.loads(value)
@@ -1504,8 +1538,8 @@ class SOPickleCol(SOBLOBCol):
         super(SOPickleCol, self).__init__(**kw)
 
     def createValidators(self):
-        return [PickleValidator(
-            name=self.name, pickleProtocol=self.pickleProtocol)] + \
+        return [PickleValidator(name=self.name,
+                pickleProtocol=self.pickleProtocol)] + \
             super(SOPickleCol, self).createValidators()
 
     def _mysqlType(self):
